#!/bin/sh

if (( $EUID != 0 )); then

    echo "macOS Install Drive Maker muss mit Administrator Rechten gestartet werden:"
    sudo "$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)/$(basename "${BASH_SOURCE[0]}")"
    exit
fi

function header
{

	clear
	echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
	echo "+                          macOS Install Drive Maker                           +"
	echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
	echo

	if [ "$setInstaller" != "" ]; then
		echo "> Gewähltes Installationsprogramm: $setInstaller"
	fi
	if [ "$setDisk" != "" ]; then
		echo "> Gewähltes Ziellaufwerk: $setDisk"
	fi
	if [ "$setMessageFormat" != "" ]; then
		echo "> Schritt 1: $setDisk formatieren"
	fi
	if [ "$setMessageMountESD" != "" ]; then
		echo "> Schritt 2: $setInstallESD einbinden"
	fi
	if [ "$setMessageCopyBaseSystem" != "" ]; then
		echo "> Schritt 3: Systembasis auf das Laufwerk kopieren"
	fi
	if [ "$setMessageCopyPackages" != "" ]; then
		echo "> Schritt 4: Pakete auf das Laufwerk kopieren"
	fi
	if [ "$setMessageCleanup" != "" ]; then
		echo "> Schritt 5: Aufräumen"
	fi
	if [ "$setMessageDone" != "" ]; then
		echo "> Fertig!"
	fi

	echo
	echo "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++"
}

function unmountInstallESD
{

	volume=$(diskutil list | grep Mac\ OS\ X\ Install\ ESD | awk '{print $10}' | cut -f 2 -d "k" | cut -f 1 -d "s")
	if [ "$volume" != "" ]; then
		diskutil unmountDisk force "disk$volume"
	fi

	volume=$(diskutil list | grep OS\ X\ Install\ ESD | awk '{print $9}' | cut -f 2 -d "k" | cut -f 1 -d "s")
	if [ "$volume" != "" ]; then
		diskutil unmountDisk force "disk$volume"
	fi

}

while [ 1 ]; do

	header
	i=0
	installer=()

	echo "Verfügbare Installationsprogramme:"
	echo

	for package in /Applications/Install\ macOS\ *.app; do
		if [[ $package != "/Applications/Install macOS *.app" ]]; then
			let i++
			installer+=("${package#/Applications/}")
			echo "($i)\t${package#/Applications/}"
		fi
	done

	for package in /Applications/Install\ OS\ X\ *.app; do
		if [[ $package != "/Applications/Install OS X *.app" ]]; then
			let i++
			installer+=("${package#/Applications/}")
			echo "($i)\t${package#/Applications/}"
		fi
	done

	echo

	if [[ $i == 0 ]]; then

		header
		echo "Es wurden keine Installationsprogramme im Programmordner gefunden"
		echo
		echo "Bitte lade ein macOS / OS X Installationsprogramm aus dem App Store"
		echo
		echo "Danke für die Verwendung von macOS Install Drive Maker"
		echo "von Thogg Niatiz - hckntsh.de"
		echo
		exit

	elif [[ $i == 1 ]]; then

		setInstaller="$(echo ${installer[0]} | cut -f 1 -d ".")"

		# early version workaround
		if [[ "${installer[0]}" == "Install OS X Snow Leopard.app" ]] || [[ "${installer[0]}" == "Install OS X Lion.app" ]] || [[ "${installer[0]}" == "Install OS X Mountain Lion.app" ]]; then
			setBaseSystem="Mac OS X Base System"
			setInstallESD="Mac OS X Install ESD"
		else
			setBaseSystem="OS X Base System"
			setInstallESD="OS X Install ESD"
		fi

		break

	else

		read -p "Wähle ein Installationsprogramm: " input
		if [[ $input != "" ]]; then

			if [[ ${#installer[@]}+1 > $input ]] && [[ $input > 0 ]]; then

				setInstaller="$(echo ${installer[$input-1]} | cut -f 1 -d ".")"

				# early version workaround
				if [[ "${installer[$input-1]}" == "Install OS X Snow Leopard.app" ]] || [[ "${installer[$input-1]}" == "Install OS X Lion.app" ]] || [[ "${installer[$input-1]}" == "Install OS X Mountain Lion.app" ]]; then
					setBaseSystem="Mac OS X Base System"
					setInstallESD="Mac OS X Install ESD"
				else
					setBaseSystem="OS X Base System"
					setInstallESD="OS X Install ESD"
				fi

				break

			fi

		fi

	fi

done

while [ 1 ]; do

	header
	disks=()

	echo "Verfügbare Ziellaufwerke:"
	echo


	for volume in /dev/disk*; do

		identifier=${volume#/dev/}
		
		if [[ "$(echo $identifier | cut -f 2 -d "k")" != *"s"* ]]; then

			i="$(echo $identifier | cut -f 2 -d "k" | cut -f 1 -d "s")"
			disks+=("$i")
			echo "($i)\t$identifier"

		else

			if [[ "$(diskutil info $identifier | grep "Volume Name:" | awk '{print $3$4}')" != "Notapplicable" ]]; then
				echo "$(diskutil info $identifier | grep "Volume Name:" | cut -f 2 -d ":")"
			fi

		fi

	done

	echo

	read -p "Wähle ein Ziellaufwerk: " input
	if [[ $input != "" ]]; then

		if [ -b "/dev/disk$input" ]; then

			echo "/dev/disk$input"
			setDisk="disk$input"
			break

		fi

	fi

done

setMessageFormat="true"
header
echo "Bereit! Das Installationslaufwerk kann jetzt erstellt werden"
echo "Dadurch werden alle Daten auf $setDisk gelöscht und durch die Installationsdateien ersetzt"
echo

read -p "Gib START ein, um fortzufahren: " input
if [[ $input != "START" ]]; then

	exit

fi


diskutil eraseDisk JHFS+ "$setInstaller" "$setDisk"

setMessageMountESD="true"
header
hdiutil attach "/Applications/$setInstaller.app/Contents/SharedSupport/InstallESD.dmg"

setMessageCopyBaseSystem="true"
header
echo "Copying. This might take a while..."
asr restore --verbose --source "/Volumes/$setInstallESD/BaseSystem.dmg" --target "/Volumes/$setInstaller" --erase --noprompt
diskutil rename "$setBaseSystem" "$setInstaller"
rsync --progress "/Volumes/$setInstallESD/BaseSystem.chunklist" "/Volumes/$setInstaller"
rsync --progress "/Volumes/$setInstallESD/BaseSystem.dmg" "/Volumes/$setInstaller"

setMessageCopyPackages="true"
header
rm -rf "/Volumes/$setInstaller/System/Installation/Packages"
echo "Copying packages. This might take a while..."
rsync -r --progress "/Volumes/$setInstallESD/Packages/" "/Volumes/$setInstaller/System/Installation/Packages/"

setMessageCleanup="true"
header
chflags -hf hidden "/Volumes/$setInstaller/"*
chflags -f nohidden "/Volumes/$setInstaller/$setInstaller.app"
rm -rf "/Volumes/$setInstaller/.fseventsd"
rm -rf "/Volumes/$setInstaller/.Spotlight-V100"
rm -rf "/Volumes/$setInstaller/.vol"
unmountInstallESD

setMessageDone="true"
header
echo
echo "Danke für die Verwendung von macOS Install Drive Maker"
echo "von Thogg Niatiz - hckntsh.de"
echo